mod data;
mod grpc;

use data::BrainData;
use detee_shared::sgx::pb::brain::brain_app_cli_server::BrainAppCliServer;
use detee_shared::sgx::pb::brain::brain_app_daemon_server::BrainAppDaemonServer;
use grpc::snp_proto::brain_cli_server::BrainCliServer;
use grpc::snp_proto::brain_vm_daemon_server::BrainVmDaemonServer;
use grpc::BrainAppCliMock;
use grpc::BrainAppDaemonMock;
use grpc::BrainCliMock;
use grpc::BrainDaemonMock;
use std::sync::Arc;
use tonic::transport::Server;

#[tokio::main]
async fn main() {
    env_logger::builder()
        .filter_level(log::LevelFilter::Debug)
        .init();
    let data = Arc::new(BrainData::new());
    let data_clone = data.clone();
    tokio::spawn(async move {
        loop {
            tokio::time::sleep(tokio::time::Duration::from_secs(60)).await;
            data_clone.vm_nodes_cron().await;
            data_clone.vm_contracts_cron().await;
            data_clone.app_contracts_cron().await;
            if let Err(e) = data_clone.save_to_disk() {
                log::error!("Could not save data to disk due to error: {e}")
            }
        }
    });
    let addr = "0.0.0.0:31337".parse().unwrap();

    let daemon_server = BrainVmDaemonServer::new(BrainDaemonMock::new(data.clone()));
    let cli_server = BrainCliServer::new(BrainCliMock::new(data.clone()));

    let sgx_cli_server = BrainAppCliServer::new(BrainAppCliMock::new(data.clone()));
    let sgx_daemon_server = BrainAppDaemonServer::new(BrainAppDaemonMock::new(data.clone()));

    Server::builder()
        .add_service(daemon_server)
        .add_service(cli_server)
        .add_service(sgx_cli_server)
        .add_service(sgx_daemon_server)
        .serve(addr)
        .await
        .unwrap();
}
